# -*- coding: utf-8 -*-
#
# Copyright (C) 2022 CERN.
#
# Invenio-Communities is free software; you can redistribute it and/or
# modify it under the terms of the MIT License; see LICENSE file for more
# details.

"""Invenio administration OAI-PMH view module."""
from invenio_administration.views.base import (
    AdminResourceDetailView,
    AdminResourceListView,
)

from invenio_communities.communities.schema import CommunityFeaturedSchema


class CommunityListView(AdminResourceListView):
    """Search admin view."""

    api_endpoint = "/communities"
    name = "communities"
    resource_config = "communities_resource"
    search_request_headers = {"Accept": "application/vnd.inveniordm.v1+json"}
    title = "Communities"
    menu_label = "Communities"
    category = "Communities"
    pid_path = "id"
    icon = "users"
    template = "invenio_communities/administration/community_search.html"

    display_search = True
    display_delete = False
    display_create = False
    display_edit = False

    item_field_list = {
        "slug": {
            "text": "Slug",
            "order": 1,
        },
        "metadata.title": {"text": "Title", "order": 2},
        # This field is for display only, it won't work on forms
        "ui.type.title_l10n": {"text": "Type", "order": 3},
        "featured": {"text": "Featured", "order": 4},
        "created": {"text": "Created", "order": 5},
    }

    actions = {
        "featured": {
            "text": "Feature",
            "payload_schema": CommunityFeaturedSchema,
            "order": 1,
        }
    }
    search_config_name = "COMMUNITIES_SEARCH"
    search_facets_config_name = "COMMUNITIES_FACETS"
    search_sort_config_name = "COMMUNITIES_SORT_OPTIONS"


class CommunityDetailView(AdminResourceDetailView):
    """Admin community detail view."""

    url = "/communities/<pid_value>"
    api_endpoint = "/communities"
    name = "community-details"
    resource_config = "communities_resource"
    title = "Community"

    template = "invenio_communities/administration/community_details.html"
    display_delete = False
    display_edit = False

    list_view_name = "communities"
    pid_path = "id"
    request_headers = {"Accept": "application/vnd.inveniordm.v1+json"}

    actions = {
        "featured": {
            "text": "Feature",
            "payload_schema": CommunityFeaturedSchema,
            "order": 1,
        }
    }

    item_field_list = {
        "slug": {
            "text": "Slug",
            "order": 1,
        },
        "metadata.title": {"text": "Title", "order": 2},
        # This field is for display only, it won't work on forms
        "ui.type.title_l10n": {"text": "Type", "order": 3},
        "created": {"text": "Created", "order": 5},
    }



# -*- coding: utf-8 -*-
#
# Copyright (C) 2022 Northwestern University.
# Copyright (C) 2022 CERN.
#
# Invenio-Communities is free software; you can redistribute it and/or modify
# it under the terms of the MIT License; see LICENSE file for more details.

"""Members Service Config."""

from flask_babelex import lazy_gettext as _
from invenio_records_resources.services import (
    RecordServiceConfig,
    SearchOptions,
    pagination_links,
)
from invenio_records_resources.services.records.components import MetadataComponent
from invenio_records_resources.services.records.queryparser import (
    QueryParser,
    SearchFieldTransformer,
)

from ...communities.records.api import Community
from ...permissions import CommunityPermissionPolicy
from ..records import Member
from ..records.api import ArchivedInvitation
from . import facets
from .components import CommunityMemberCachingComponent
from .schemas import MemberEntitySchema


class PublicSearchOptions(SearchOptions):
    """Public Search Options."""

    sort_default = "bestmatch"
    sort_default_no_query = "name"
    sort_options = {
        "bestmatch": dict(
            title=_("Best match"),
            fields=["_score"],  # ES defaults to desc on `_score` field
        ),
        "name": dict(
            title=_("Name"),
            fields=["user.profile.full_name.keyword"],
        ),
    }

    query_parser_cls = QueryParser.factory(
        # fields serves also as a filter, since only the ones mentioned there
        # will be queried by the search engine, using multi_match query.
        fields=[
            "user.username^2",
            "user.profile.full_name^3",
            "user.profile.affiliations",
        ],
        tree_transformer_factory=SearchFieldTransformer.factory(
            mapping={
                "affiliation": "user.profile.affiliations",
                "affiliations": "user.profile.affiliations",
                "full_name": "user.profile.full_name",
                "fullname": "user.profile.full_name",
                "name": "user.profile.full_name",
                "username": "user.username",
            }
        ),
    )


class InvitationsSearchOptions(SearchOptions):
    """Invitations Search Options."""

    # TODO: should restrict fields that's being searched on.
    # query_parser_cls = QueryParser
    sort_default = "bestmatch"
    sort_default_no_query = "name"
    sort_options = {
        "bestmatch": dict(
            title=_("Best match"),
            fields=["_score"],  # ES defaults to desc on `_score` field
        ),
        "name": dict(
            title=_("Name"),
            fields=["user.profile.full_name.keyword"],
        ),
        "newest": dict(
            title=_("Newest"),
            fields=["-created"],
        ),
        "oldest": dict(
            title=_("Oldest"),
            fields=["created"],
        ),
    }

    facets = {
        "role": facets.role,
        "status": facets.status,
        "is_open": facets.is_open,
    }


class MemberSearchOptions(PublicSearchOptions):
    """Search options."""

    sort_default = "bestmatch"
    sort_default_no_query = "name"
    sort_options = {
        "bestmatch": dict(
            title=_("Best match"),
            fields=["_score"],  # ES defaults to desc on `_score` field
        ),
        "name": dict(
            title=_("Name"),
            fields=["user.profile.full_name.keyword"],
        ),
        "newest": dict(
            title=_("Newest"),
            fields=["-created"],
        ),
        "oldest": dict(
            title=_("Oldest"),
            fields=["created"],
        ),
    }

    facets = {
        "role": facets.role,
        "visibility": facets.visibility,
    }

    query_parser_cls = QueryParser.factory(
        fields=[
            "user.username^2",
            "user.email^2",
            "user.profile.full_name^3",
            "user.profile.affiliations",
        ],
        tree_transformer_factory=SearchFieldTransformer.factory(
            mapping={
                "affiliation": "user.profile.affiliations",
                "affiliations": "user.profile.affiliations",
                "email": "user.email",
                "full_name": "user.profile.full_name",
                "fullname": "user.profile.full_name",
                "name": "user.profile.full_name",
                "username": "user.username",
            }
        ),
    )


class MemberServiceConfig(RecordServiceConfig):
    """Member Service Config."""

    service_id = "members"

    community_cls = Community
    record_cls = Member
    schema = MemberEntitySchema
    indexer_queue_name = "members"
    relations = {"users": ["user"]}

    archive_cls = ArchivedInvitation
    archive_indexer_cls = RecordServiceConfig.indexer_cls
    archive_indexer_queue_name = "archived-invitations"

    permission_policy_cls = CommunityPermissionPolicy

    search = MemberSearchOptions
    search_public = PublicSearchOptions
    search_invitations = InvitationsSearchOptions

    # No links
    links_item = {}

    # ResultList configurations
    links_search = pagination_links("{+api}/communities/{community_id}/members{?args*}")

    # Service components
    components = [
        MetadataComponent,
        CommunityMemberCachingComponent,
    ]



# -*- coding: utf-8 -*-
#
# Copyright (C) 2022 Graz University of Technology.
#
# Invenio-Records-Resources is free software; you can redistribute it and/or
# modify it under the terms of the MIT License; see LICENSE file for more
# details.

"""Unit of work operations for featured community services."""

from invenio_db import db
from invenio_records_resources.services.uow import Operation


class CommunityFeaturedCommitOp(Operation):
    """Featured community add/update operation."""

    def __init__(self, featured_entry):
        """Initialize the commit operation."""
        super().__init__()
        self._featured_entry = featured_entry

    def on_register(self, uow):
        """Add to db session."""
        db.session.add(self._featured_entry)


class CommunityFeaturedDeleteOp(Operation):
    """OAI-PMH set delete operation."""

    def __init__(self, featured_entry):
        """Initialize the delete operation."""
        super().__init__()
        self._featured_entry = featured_entry

    def on_register(self, uow):
        """Delete entry."""
        db.session.delete(self._featured_entry)



# -*- coding: utf-8 -*-
#
# This file is part of Invenio.
# Copyright (C) 2016-2022 CERN.
# Copyright (C) 2021 Graz University of Technology.
# Copyright (C) 2021 TU Wien.
# Copyright (C) 2022 Northwestern University.
#
# Invenio is free software; you can redistribute it and/or modify it
# under the terms of the MIT License; see LICENSE file for more details.

"""Community permissions."""

from invenio_administration.generators import Administration
from invenio_records_permissions.generators import (
    AnyUser,
    AuthenticatedUser,
    SystemProcess,
)
from invenio_records_permissions.policies import BasePermissionPolicy

from .generators import (
    AllowedMemberTypes,
    CommunityCurators,
    CommunityManagers,
    CommunityManagersForRole,
    CommunityMembers,
    CommunityOwners,
    CommunitySelfMember,
    GroupsEnabled,
    IfPolicyClosed,
    IfRestricted,
)


# Permission Policy
class CommunityPermissionPolicy(BasePermissionPolicy):
    """Permissions for Community CRUD operations."""

    # Community
    can_create = [AuthenticatedUser(), SystemProcess()]

    can_read = [
        IfRestricted("visibility", then_=[CommunityMembers()], else_=[AnyUser()]),
        SystemProcess(),
    ]

    can_update = [CommunityOwners(), SystemProcess()]

    can_delete = [CommunityOwners(), SystemProcess()]

    can_search = [AnyUser(), SystemProcess()]

    can_search_user_communities = [AuthenticatedUser(), SystemProcess()]

    can_search_invites = [CommunityManagers(), SystemProcess()]

    can_search_requests = [CommunityManagers(), CommunityCurators(), SystemProcess()]

    can_rename = [CommunityOwners(), SystemProcess()]

    can_submit_record = [
        IfPolicyClosed(
            "record_policy",
            then_=[CommunityMembers(), SystemProcess()],
            else_=[
                IfRestricted(
                    "visibility",
                    then_=[CommunityMembers()],
                    else_=[AuthenticatedUser()],
                ),
            ],
        ),
    ]

    can_members_add = [
        CommunityManagersForRole(),
        AllowedMemberTypes("group"),
        GroupsEnabled("group"),
        SystemProcess(),
    ]

    can_members_invite = [
        CommunityManagersForRole(),
        AllowedMemberTypes("user", "email"),
        SystemProcess(),
    ]

    can_members_manage = [
        CommunityManagers(),
        SystemProcess(),
    ]

    can_members_search = [
        CommunityMembers(),
        SystemProcess(),
    ]

    can_members_search_public = [
        IfRestricted("visibility", then_=[CommunityMembers()], else_=[AnyUser()]),
        SystemProcess(),
    ]

    # Ability to use membership update api
    can_members_bulk_update = [
        CommunityMembers(),
        SystemProcess(),
    ]

    can_members_bulk_delete = can_members_bulk_update

    # Ability to update a single membership
    can_members_update = [
        CommunityManagersForRole(),
        CommunitySelfMember(),
        SystemProcess(),
    ]

    # Ability to delete a single membership
    can_members_delete = can_members_update

    can_invite_owners = [CommunityOwners(), SystemProcess()]

    # Abilities for featured communities
    can_featured_search = [AnyUser(), SystemProcess()]
    can_featured_list = [SystemProcess()]
    can_featured_create = [Administration(), SystemProcess()]
    can_featured_update = [Administration(), SystemProcess()]
    can_featured_delete = [Administration(), SystemProcess()]


def can_perform_action(community, context):
    """Check if the given action is available on the request."""
    action = context.get("action")
    identity = context.get("identity")
    permission_policy_cls = context.get("permission_policy_cls")
    permission = permission_policy_cls(action, community=community)
    return permission.allows(identity)



# -*- coding: utf-8 -*-
#
# Copyright (C) 2022 CERN.
#
# Invenio-RDM-Records is free software; you can redistribute it and/or modify
# it under the terms of the MIT License; see LICENSE file for more details.

"""Facet definitions."""

from flask_babelex import gettext as _
from invenio_records_resources.services.records.facets import TermsFacet

type = TermsFacet(
    field="metadata.type.id",
    label=_("Type"),
    value_labels={
        "organization": _("Organization"),
        "event": _("Event"),
        "topic": _("Topic"),
        "project": _("Project"),
    },
)

visibility = TermsFacet(
    field="access.visibility",
    label=_("Visibility"),
    value_labels={
        "public": _("Public"),
        "restricted": _("Restricted"),
    },
)

role = TermsFacet(
    field="role",
    label=_("Visibility"),
    value_labels={
        "owner": _("Owner"),
        "reader": _("Reader"),
        "manager": _("Manager"),
        "curator": _("Curator"),
    },
)

visible = TermsFacet(
    field="visible",
    label=_("Visibility"),
    value_labels={
        "hidden": _("Hidden"),
        "restricted": _("Restricted"),
    },
)
